/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.file.storage.composition.internal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.util.tracker.ServiceTracker;
import com.openexchange.file.storage.composition.FileStreamHandlerRegistry;
import com.openexchange.file.storage.composition.FileStreamHandler;

/**
 * {@link FileStreamHandlerRegistryImpl}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class FileStreamHandlerRegistryImpl extends ServiceTracker<FileStreamHandler, FileStreamHandler> implements FileStreamHandlerRegistry {

    private final ConcurrentMap<Class<? extends FileStreamHandler>, FileStreamHandler> registry;
    private final Comparator<FileStreamHandler> comparator;

    /**
     * Initializes a new {@link FileStreamHandlerRegistryImpl}.
     *
     * @param context The bundle context
     */
    public FileStreamHandlerRegistryImpl(final BundleContext context) {
        super(context, FileStreamHandler.class, null);
        registry = new ConcurrentHashMap<Class<? extends FileStreamHandler>, FileStreamHandler>(8, 0.9f, 1);
        comparator = new Comparator<FileStreamHandler>() {

            @Override
            public int compare(final FileStreamHandler o1, final FileStreamHandler o2) {
                final int r1 = o1 == null ? 0 : o1.getRanking();
                final int r2 = o2 == null ? 0 : o2.getRanking();
                return (r1 < r2 ? 1 : (r1 == r2 ? 0 : -1));
            }
        };
    }

    @Override
    public FileStreamHandler addingService(final ServiceReference<FileStreamHandler> reference) {
        final FileStreamHandler streamHandler = super.addingService(reference);
        if (null == registry.putIfAbsent(streamHandler.getClass(), streamHandler)) {
            return streamHandler;
        }
        context.ungetService(reference);
        return null;
    }

    @Override
    public void removedService(final ServiceReference<FileStreamHandler> reference, final FileStreamHandler streamHandler) {
        if (null != streamHandler) {
            registry.remove(streamHandler.getClass());
            super.removedService(reference, streamHandler);
        }
    }

    @Override
    public Collection<FileStreamHandler> getHandlers() {
        final List<FileStreamHandler> ret = new ArrayList<FileStreamHandler>(registry.values());
        Collections.sort(ret, comparator);
        return ret;
    }

}
